@@ -1,6 +1,7 @@
 #ifndef GLMMR_H
 #define GLMMR_H
 
+#define EIGEN_PERMANENTLY_DISABLE_STUPID_WARNINGS 
 
 #include "glmmr/general.h"
 #include "glmmr/maths.h"
@@ -8,6 +9,8 @@
 #include "glmmr/covariance.hpp"
 #include "glmmr/linearpredictor.hpp"
 #include "glmmr/model.hpp"
+#include "glmmr/modelbits.hpp"
+#include "glmmr/openmpheader.h"
 #include <RcppEigen.h>
 
 // [[Rcpp::depends(RcppEigen)]]
